/****** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XpcomViewer code.
 *
 * The Initial Developer of the Original Code is NeutronStar, s.r.o.
 * Portions created by NeutronStar, s.r.o. are Copyright (C) 2005-2006
 * NeutronStar, s.r.o.. All Rights Reserved.
 *
 * Contributor(s):
 *  - Ondrej Donek, <ondrejd@gmail.com>, original author
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 ****** END LICENSE BLOCK *****/



/**
 * Main gXpcomViewer class.
 */
var gXpcomViewer =
{

	/**
	 * When XPCOMView application window started.
	 */
	onLoad : function ()
	{
    window.focus ();

    // Initialize UI pointers
		gStrBundle = document.getElementById ("stringBundle");
		gBrandStrBundle = document.getElementById ("brandStringBundle");
    gStatusbarLabel = document.getElementById ("status-label");
    gStatusbarIcon = document.getElementById ("status-icon");
		gXpcomTree = document.getElementById ("xpcom-tree");

    // Initalize bookmarks (if user want)
    try {
	    if ((kPrefs.getBoolPref ("allowBookmarks"))) {
	      startBookmarkService ();
	    } else {
	      disableBookmarkService ();
	    }
	  } catch (e) {
	    Components.utils.reportError (e);
      disableBookmarkService ();
	  }

    // Initialize notes (if user want)
    try {
	    if ((kPrefs.getBoolPref ("allowNotes"))) {
	      startNoteService ();
	    } else {
	      disableNoteService ();
	    }
	  } catch (e) {
	    Components.utils.reportError (e);
		  disableNoteService ();
	  }

		// Restore last or use custom view (according to user prefs.)
		var lastView = kDefaultWorkspaceView;
		try {
			if ((kPrefs.getBoolPref ("useLastView")) &&
			   (kPrefs.prefHasUserValue ("useLastViewName"))) {
			  lastView = kPrefs.getCharPref ("useLastViewName");
			}
	  } catch (e) {
	    alert ("Error when loading user preferences");
			Components.utils.reportError (e);
	    lastView = kDefaultWorkspaceView;
	  }

    // Refresh XpcomViewer tree
    this.refreshTree (lastView);
	}, // onLoad ()


	/**
	 * Fired before onUnLoad -> here we're saving our datasources becouse of
	 * this is the last time when is app. window visible so we can use our
	 * progress meter.
	 * @todo    There should be any progress meter when
	 * @todo    Error dialog can contains checkbox if user want see this
	 *          dialog every time when error occured
	 */
	onClose : function ()
	{
    // XXX Save last used view and it's filter if is any applied
    gStatusbar.updateLabel ("Saving last used tree view.");
    try {
	    if ((kPrefs.getBoolPref ("useLastView"))) {
	      kPrefs.setCharPref ("useLastViewName", gCurrentTreeView);
	    }
	  } catch (e) {
	    alert ("Error when saving last used view.");
			Components.utils.reportError (e);
	  }

	  // XXX Save filters (if any)

	  return true;
	}, // onClose ()


	/**
	 * On gXpcomViewer unload. This function only can finish pendling
	 * operations but can not stop closing application!
	 */
	onUnLoad : function ()
	{
    return true;
	}, // end onUnLoad ()


	/**
	 * When tree item is selected.
	 */
	onTreeSelect : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {
	    gStatusbar.updateLabel ("Selected item: " +
	    		gXpcomTreeView[gXpcomTree.currentIndex].mNameCol);
	  }
	}, // end onTreeSelect (aEvent)


	/**
	 * When user double clicked on the three.
	 */
	onTreeDblClick : function (aEvent)
	{
	  gStatusbar.updateLabel ("You can attach note or bookmark through the "+
	      "tree context menu.");
	}, // end onTreeDblClick (aEvent)


	/**
	 * Fired when bookmarks toolbarbutton's popupmenu is showing.
	 *
	 * @todo Count of displayed items should be set up by user
	 */
	onBookmarkPopupShowing : function ()
	{
	  var popup = document.getElementById ("bookmarkBtn-popup");
	  var bookmarks = gBookmarksDataSource.getAllBookmarks (15);

	  if (bookmarks.length == 0) {
	    var menuitem = document.createElementNS (XUL_XMLNS, "menuitem", "");

	    menuitem.setAttribute ("label", "There are no bookmarks");
	    menuitem.setAttribute ("style", "font-style:italic;");
	    popup.appendChild (menuitem);

	    return;
	  }

	  for (var i=0; i<bookmarks.length; i++) {
	    var menuitem = document.createElementNS (XUL_XMLNS, "menuitem", "");

	    menuitem.setAttribute ("label", bookmarks[i]);
	    popup.appendChild (menuitem);
	  }
	}, // end onBookmarkPopupShowing ()


	/**
	 * Fired when bookmarks toolbarbutton's popupmenu is hiding.
	 */
	onBookmarkPopupHiding : function ()
	{
	  var popup = document.getElementById ("bookmarkBtn-popup");
	  var menuitemsToRemove = [];

	  for (var i=0;i<popup.childNodes.length;i++) {
	    if (popup.childNodes[i].id != "bookmarkBtnPopupAddCmd" &&
	        popup.childNodes[i].id != "bookmarkBtnPopupSep") {
	      menuitemsToRemove.push (popup.childNodes[i]);
	    }
	  }

	  for (var menuitem in menuitemsToRemove) {
	    popup.removeChild (menuitemsToRemove [menuitem]);
	  }
	}, // end onBookmarkPopupHiding ()


	/**
	 * Fired when notes toolbarbutton's popupmenu is showing.
	 *
	 * @todo Count of displayed items should be set up by user
	 */
	onNotePopupShowing : function ()
	{
	  var popup = document.getElementById ("noteBtn-popup");
	  var notes = gNotesDataSource.getAllNotes (15);

	  if (notes.length == 0) {
	    var menuitem = document.createElementNS (XUL_XMLNS, "menuitem", "");

	    menuitem.setAttribute ("label", "There are no notes");
	    menuitem.setAttribute ("style", "font-style:italic;");
	    popup.appendChild (menuitem);

	    return;
	  }

	  for (var i=0; i<notes.length; i++) {
	    var menuitem = document.createElementNS (XUL_XMLNS, "menuitem", "");

	    menuitem.setAttribute ("label", notes[i]);
	    popup.appendChild (menuitem);
	  }
	}, // end onNotePopupShowing ()


	/**
	 * Fired when notes toolbarbutton's popupmenu is hiding.
	 */
	onNotePopupHiding : function ()
	{
	  var popup = document.getElementById ("noteBtn-popup");
	  var menuitemsToRemove = [];

	  for (var i=0;i<popup.childNodes.length;i++) {
	    if (popup.childNodes[i].id != "noteBtnPopupAddCmd" &&
	        popup.childNodes[i].id != "noteBtnPopupSep") {
	      menuitemsToRemove.push (popup.childNodes[i]);
	    }
	  }

	  for (var menuitem in menuitemsToRemove) {
	    popup.removeChild (menuitemsToRemove [menuitem]);
	  }
	}, // end onNotePopupHiding ()


	/**
	 * Refresh tree with given reference uri.
	 *
	 * @param  string
	 */
	refreshTree : function (aTreeType)
	{
    gCurrentTreeView = aTreeType;
    gXpcomTreeView = [];

    // Remove filter if any applied
    if (gXpcomTreeFilter != "none") this.removeFilter ();

    // Restore wanted treeview from backup (if treeview was created before)
    var restoredTreeView = false;
    if (gXpcomTreeViewBackup[aTreeType].length != 0)
      restoredTreeView = gXpcomTreeViewBackup[aTreeType];

    // Create (if restoring failed) treeview and initialize tree
	  switch (aTreeType)
	  {
	    case "classes": // Name is Contract ID | Value is Class ID
	      gXpcomClassesCount = 0;

	      if (!(restoredTreeView)) {
				  for (var cl in Components.classes) {
				    var cls = Components.classes[cl];

				    if (!cls) continue;

				    gXpcomTreeView.push (new newTreeItem (cls.name, cls.number, "class", 0, true));
				    gXpcomTreeView[gXpcomClassesCount].mIsContainerEmpty = false;
				    gXpcomClassesCount++;
				  }
				  gXpcomTreeViewBackup.classes = gXpcomTreeView;
				} else {
				  gXpcomTreeView = restoredTreeView;
				}

				gXpcomTree.view = gTreeClassesPrototype;
	      break;

	    case "ifaces":
	      gXpcomIfacesCount = 0;

	      if (!(restoredTreeView)) {
					for (var ifc in Components.interfaces) {
					  var iface = Components.interfaces[ifc];

					  gXpcomTreeView.push (new newTreeItem (iface.name, iface.number,
					      "interface", 0, false));

	          var addIndex = 0;
					  for (var ifaceItem in iface) addIndex++;

					  if (addIndex > 0) {
					    gXpcomTreeView[gXpcomIfacesCount].mIsContainer = true;
					    gXpcomTreeView[gXpcomIfacesCount].mIsContainerEmpty = false;
					  }

					  gXpcomIfacesCount++;
					}
				  gXpcomTreeViewBackup.ifaces = gXpcomTreeView;
				} else {
				  gXpcomTreeView = restoredTreeView;
				}

				gXpcomTree.view = gTreeIfacesPrototype;
	      break;

	    case "results":
	      gXpcomResultsCount = 0;

	      if (!(restoredTreeView)) {
	        for (var res in Components.results) {
	          gXpcomTreeView.push (new newTreeItem (res, Components.results[res],
	              "result", 0, false));

	          gXpcomResultsCount++;
	        }
				  gXpcomTreeViewBackup.results = gXpcomTreeView;
				} else {
				  gXpcomTreeView = restoredTreeView;
				}

		  	gXpcomTree.view = gTreeResultsPrototype;
	      break;
	  }

		gStatusbar.updateLabel ("Loading "+kTreeViewsNames[gCurrentTreeView]+" finished "+
		    "[loaded "+gXpcomTreeView.length+" items]");
		document.title = "XPCOM Viewer - "+kTreeViewsNames[gCurrentTreeView]+" ["+
		    gXpcomTreeView.length+"]";
	}, // end refreshTree (aTreeType)


	/**
	 * Apply filter on XPCOM datasource and refresh the main
	 * tree.
	 *
	 * @param   string
	 */
	applyFilter : function (aFilterType)
	{
	  var applyFilterBtn = document.getElementById ("filter-button");
	  var removeFilterBtn = document.getElementById ("removeFilter-button");
	  var newXpcomTreeView = [];
	  var treeFilterBroadcaster = document.getElementById ("removeFilter-broadcaster");

	  // Perform filter action
	  if ((aFilterType == "containing") || (aFilterType == "starting_with")) {
      var searchString = { value : "" };
      var caseSensitive = { value : false };
      var res = kPromptSrv.prompt (null,
          "XPCOMViewer - set '"+aFilterType+"' filter",
          "Insert string you looking for:", searchString,
          "Case sensitive?", caseSensitive);

			if (!(res) || searchString.value == "") {
			  gStatusbar.updateLabel ("No string for filter inserted. Canceling ...");
			  return;
			}

			caseSensitive = caseSensitive.value;
			searchString = searchString.value;
			searchString = (!(caseSensitive)) ? searchString.toLowerCase (): searchString;

      for (var item in gXpcomTreeView) {
        if (aFilterType == "containing") {
	        if (gXpcomTreeView[item].mNameCol.indexOf (searchString) != -1)
	          newXpcomTreeView.push (gXpcomTreeView[item]);
	      } else {
	        if (gXpcomTreeView[item].mNameCol.indexOf (searchString) == 0)
	          newXpcomTreeView.push (gXpcomTreeView[item]);
	      }
      }
	  } else if ((aFilterType == "with_note") || (aFilterType == "with_bookmark")) {
      for (var item in gXpcomTreeView) {
        if (aFilterType == "with_note" && gXpcomTreeView[item].mNoteCol == true) {
	        newXpcomTreeView.push (gXpcomTreeView[item]);
        } else if (aFilterType == "with_bookmark" && gXpcomTreeView[item].mBookmarkCol == true) {
	        newXpcomTreeView.push (gXpcomTreeView[item]);
        }
      }
	  } else {
	    gStatusbar.updateLabel ("Bad filter type. Canceling ...");
	    return;
	  }

	  // Update tree view
	  if (newXpcomTreeView.length == 0) {
	    gStatusbar.updateLabel ("No items match the filter pattern.");
	  } else {
	    gXpcomTreeFilter = aFilterType;
	    gXpcomTreeFilterBackup = gXpcomTreeView;
		  gXpcomTreeView = newXpcomTreeView;

		  switch (gCurrentTreeView) {
		    case "classes":
		      document.title = "XPCOMViewer - Components.classes (filter) [" +
		          gXpcomTreeView.length + "]";
		      gXpcomTree.view = gTreeClassesPrototype;
		      break;
		    case "interfaces":
		      document.title = "XPCOMViewer - Components.interfaces (filter) [" +
		          gXpcomTreeView.length + "]";
		      gXpcomTree.view = gTreeIfacesPrototype;
		      break;
		    case "results":
		      document.title = "XPCOMViewer - Components.results (filter) [" +
		          gXpcomTreeView.length + "]";
		      gXpcomTree.view = gTreeResultsPrototype;
		      break;
		  }

	    gStatusbar.updateLabel (newXpcomTreeView.length + " items match the filter pattern.");
      applyFilterBtn.setAttribute ("disabled", true);
      if (removeFilterBtn.hasAttribute ("disabled"))
        removeFilterBtn.removeAttribute ("disabled");
		}
	}, // end applyFilter ()


	/**
	 * Remove any applied filter.
	 */
	removeFilter : function ()
	{
	  var applyFilterBtn = document.getElementById ("filter-button");
	  var removeFilterBtn = document.getElementById ("removeFilter-button");

    gXpcomTreeFilter = "none";
	  gXpcomTreeView = gXpcomTreeFilterBackup;

	  switch (gCurrentTreeView) {
	    case "classes":
	      document.title = "XPCOMViewer - Components.classes [" +
	          gXpcomTreeView.length + "]";
	      gXpcomTree.view = gTreeClassesPrototype;
	      break;
	    case "interfaces":
	      document.title = "XPCOMViewer - Components.interfaces [" +
	          gXpcomTreeView.length + "]";
	      gXpcomTree.view = gTreeIfacesPrototype;
	      break;
	    case "results":
	      document.title = "XPCOMViewer - Components.results [" +
	          gXpcomTreeView.length + "]";
	      gXpcomTree.view = gTreeResultsPrototype;
	      break;
	  }

    gStatusbar.updateLabel (gStrBundle.getString ("filterRemovedMsg"));
	  removeFilterBtn.setAttribute ("disabled", true);
	  if (applyFilterBtn.hasAttribute ("disabled"))
	    applyFilterBtn.removeAttribute ("disabled");
	}, // end removeFilter ()


	/**
	 * Add bookmark command launched from toolbar or tree context menu
	 */
	addBookmark : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {
      if (!(gXpcomTreeView[gXpcomTree.currentIndex].mBookmarkCol)) {
        var res = gBookmarksDataSource.add (gXpcomTreeView[gXpcomTree.currentIndex].mNameCol);
        if (!(res)) {
          alert ("Bookmark adding failed!\nPlease, try again.");
          return;
        }

        gXpcomTreeView[gXpcomTree.currentIndex].mBookmarkCol = true;
      } else {
        gStatusBar.updateLabel ("Selected item already has bookmark.");
      }
	  } else {
	    gStatusbar.updateLabel ("\"Add bookmark\" failed! No item is selected ...");
	  }
	}, // end addBookmark (aEvent)


	/**
	 * Remove bookmark command launched from tree context menu
	 */
	removeBookmark : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {
      if ((gXpcomTreeView[gXpcomTree.currentIndex].mBookmarkCol)) {
        var res = gBookmarksDataSource.remove (gXpcomTreeView[gXpcomTree.currentIndex].mNameCol);
        if (!(res)) {
          alert ("Bookmark removing failed!\nPlease, try again.");
          return;
        }

        gXpcomTreeView[gXpcomTree.currentIndex].mBookmarkCol = false;
      } else {
        gStatusBar.updateLabel ("\"Remove bookmark\" failed! Selected item hasn't bookmark.");
      }
	  } else {
	    gStatusbar.updateLabel ("\"Remove bookmark\" failed! No item is selected ...");
	  }
	}, // end removeBookmark (aEvent)


	/**
	 * Add note command launched from toolbar or tree context menu
	 */
	addNote : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {
      if (!(gXpcomTreeView[gXpcomTree.currentIndex].mNoteCol)) {
        var res = gNotesDataSource.add (gXpcomTreeView[gXpcomTree.currentIndex].mNameCol);
        if (!(res)) {
          alert ("Note adding failed!\nPlease, try again.");
          return;
        }

        gXpcomTreeView[gXpcomTree.currentIndex].mNoteCol = true;
      } else {
        gStatusBar.updateLabel ("Selected item already has note.");
      }
	  } else {
	    gStatusbar.updateLabel ("\"Add note\" failed! No item is selected ...");
	  }
	}, // end addNote (aEvent)


	/**
	 * Edit note command launched from tree context menu
	 */
	editNote : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {

	  } else {
	    gStatusbar.updateLabel ("\"Edit note\" failed! No item is selected ...");
	  }
	}, // end editNote (aEvent)


	/**
	 * Remove note command launched from tree context menu
	 */
	removeNote : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {
			if ((gXpcomTreeView[gXpcomTree.currentIndex].mNoteCol)) {
        var res = gNotesDataSource.remove (gXpcomTreeView[gXpcomTree.currentIndex].mNameCol);
        if (!(res)) {
          alert ("Note removing failed!\nPlease, try again.");
          return;
        }

        gXpcomTreeView[gXpcomTree.currentIndex].mNoteCol = false;
      } else {
        gStatusBar.updateLabel ("\"Remove note\" failed! Selected item hasn't note.");
      }
	  } else {
	    gStatusbar.updateLabel ("\"Remove note\" failed! No item is selected ...");
	  }
	}, // end removeNote (aEvent)


	/**
	 * Copy name of selected tree item (if any) to the system clipboard
	 */
	copyNameOfSelected : function (aEvent)
	{
	  if (gXpcomTree.currentIndex in gXpcomTreeView) {
			var clip = Components.classes ['@mozilla.org/widget/clipboard;1'].
			    createInstance (Components.interfaces.nsIClipboard);

			if (!clip) return;

			var trans = Components.classes ['@mozilla.org/widget/transferable;1'].
			    createInstance (Components.interfaces.nsITransferable);

			if (!trans) return;

			trans.addDataFlavor ('text/unicode');

			var str = new Object ();
			var str = Components.classes ["@mozilla.org/supports-string;1"].
			    createInstance (Components.interfaces.nsISupportsString);
			var copytext = gXpcomTreeView[gXpcomTree.currentIndex].mNameCol;

			str.data = copytext;
			trans.setTransferData ("text/unicode", str, copytext.length*2);

			clip.setData (trans, null, Components.interfaces.nsIClipboard.kGlobalClipboard);
	  } else {
	    gStatusbar.updateLabel ("\"Copy name of selected\" failed! No item is selected ...");
	  }
	}, // end copyNameOfSelected (aEvent)


	/**
	 * Copy C++ initialization code to the system clipboard
	 */
	copyCppInitCodeOfSel : function (aEvent)
	{
		alert ("Sorry, not implemented yet!");
	}, // end copyCppInitCodeOfSel (aEvent)


	/**
	 * Copy C++ initialization code to the system clipboard
	 */
	copyJSInitCodeOfSel : function (aEvent)
	{
  	alert ("Sorry, not implemented yet!");
	} // end copyJSInitCodeOfSel (aEvent)

}; // end of gXpcomViewer
